from datetime import timedelta

from data_gateway_sdk.protocols.smpm.smpm_ul_device_water_meter_16b_daily import SmpmUlDeviceWaterMeter16BDailyData
from data_gateway_sdk.utils.buf_ref import BufRef


def test_smpm_ul_device_water_meter_16b_daily() -> None:
    case_serialized = bytes.fromhex("830cc0ffff7fba90e4eab10623720d08")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=33554431.0), temperature=23, battery_volts=3.3, event_reset=False, event_low_battery_level=False, event_temperature_limits=True, direct_flow_volume=112323.3, direct_flow_volume_day_ago=3.5, reverse_flow_volume=2.788, event_battery_warn=True, event_system_error=False, event_flow_reverse=False, event_flow_speed_is_over_limit=False, event_sensor_error=False, event_sensor_error_temperature=False, event_case_was_opened=False, event_continuous_consumption=False, event_no_resource=True, event_magnet=False) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=33554431.0), temperature=23, battery_volts=3.3, event_reset=False, event_low_battery_level=False, event_temperature_limits=True, direct_flow_volume=112323.3, direct_flow_volume_day_ago=3.5, reverse_flow_volume=112323.3, event_battery_warn=True, event_system_error=False, event_flow_reverse=False, event_flow_speed_is_over_limit=False, event_sensor_error=False, event_sensor_error_temperature=False, event_case_was_opened=False, event_continuous_consumption=False, event_no_resource=True, event_magnet=False))  # noqa: E501
    case_serialized = bytes.fromhex("830c000000007fe0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c000000007fe0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000000e0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000000e0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c380000007fe0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c380000007fe0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000000e0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000000e0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c000000007fe00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c000000007fe00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000000e00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000000e00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c380000007fe00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c380000007fe00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000000e00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000000e00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c000000007fe0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c000000007fe0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000000e0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000000e0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c380000007fe0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c380000007fe0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000000e0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000000e0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c000000007fe000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c000000007fe000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000000e000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000000e000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c380000007fe000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c380000007fe000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000000e000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000000e000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c000000007fe0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c000000007fe0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000000e0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000000e0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c380000007fe0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c380000007fe0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000000e0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000000e0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c000000007fe00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c000000007fe00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000000e00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000000e00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c380000007fe00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c380000007fe00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000000e00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000000e00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c000000007fe0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c000000007fe0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000000e0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000000e0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c380000007fe0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c380000007fe0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000000e0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000000e0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c000000007fe000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c000000007fe000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000000e000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000000e000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c380000007fe000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c380000007fe000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000000e000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000000e000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc070000007fe0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc070000007fe0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000000e0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000000e0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f0000007fe0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f0000007fe0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000000e0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000000e0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc070000007fe00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc070000007fe00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000000e00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000000e00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f0000007fe00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f0000007fe00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000000e00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000000e00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc070000007fe0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc070000007fe0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000000e0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000000e0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f0000007fe0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f0000007fe0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000000e0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000000e0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc070000007fe000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc070000007fe000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000000e000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000000e000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f0000007fe000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f0000007fe000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000000e000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000000e000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc070000007fe0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc070000007fe0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000000e0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000000e0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f0000007fe0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f0000007fe0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000000e0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000000e0000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc070000007fe00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc070000007fe00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000000e00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000000e00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f0000007fe00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f0000007fe00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000000e00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000000e00000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc070000007fe0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc070000007fe0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000000e0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000000e0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f0000007fe0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f0000007fe0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000000e0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000000e0000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc070000007fe000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc070000007fe000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000000e000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000000e000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f0000007fe000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f0000007fe000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000000e000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000000e000000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c00000000ffff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c00000000ffff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000080ff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000080ff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c38000000ffff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c38000000ffff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000080ff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000080ff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c00000000ffff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c00000000ffff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000080ff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000080ff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c38000000ffff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c38000000ffff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000080ff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000080ff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c00000000ffff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c00000000ffff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000080ff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000080ff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c38000000ffff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c38000000ffff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000080ff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000080ff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c00000000ffff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c00000000ffff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000080ff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000080ff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c38000000ffff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c38000000ffff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000080ff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000080ff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c00000000ffff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c00000000ffff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000080ff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000080ff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c38000000ffff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c38000000ffff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000080ff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000080ff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c00000000ffff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c00000000ffff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000080ff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000080ff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c38000000ffff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c38000000ffff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000080ff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000080ff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c00000000ffff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c00000000ffff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000080ff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000080ff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c38000000ffff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c38000000ffff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000080ff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000080ff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c00000000ffff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c00000000ffff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000080ff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c0000000080ff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c38000000ffff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c38000000ffff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000080ff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("830c3800000080ff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc07000000ffff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc07000000ffff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000080ff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000080ff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f000000ffff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f000000ffff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000080ff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000080ff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc07000000ffff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc07000000ffff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000080ff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000080ff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f000000ffff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f000000ffff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000080ff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000080ff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc07000000ffff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc07000000ffff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000080ff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000080ff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f000000ffff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f000000ffff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000080ff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000080ff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc07000000ffff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc07000000ffff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000080ff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000080ff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f000000ffff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f000000ffff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000080ff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000080ff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc07000000ffff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc07000000ffff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000080ff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000080ff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f000000ffff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f000000ffff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000080ff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000080ff000000000000f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc07000000ffff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc07000000ffff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000080ff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000080ff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f000000ffff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f000000ffff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000080ff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000080ff0000000080ffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=0.0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc07000000ffff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc07000000ffff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000080ff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000080ff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f000000ffff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f000000ffff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000080ff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000080ff000000007f00f81f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=0.0, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc07000000ffff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc07000000ffff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000080ff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc0700000080ff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f000000ffff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f000000ffff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000080ff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("83cc3f00000080ff00000000ffffff1f")  # overflow
    assert SmpmUlDeviceWaterMeter16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=12.7, reverse_flow_volume=4.095, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, event_magnet=True) == SmpmUlDeviceWaterMeter16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeter16BDailyData.serialize(SmpmUlDeviceWaterMeter16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, direct_flow_volume_day_ago=12.7, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_magnet=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, reverse_flow_volume=8.191, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
