from datetime import timedelta

from data_gateway_sdk.protocols.smpm.smpm_ul_device_water_meter_nbiot_16b_daily import SmpmUlDeviceWaterMeterNbiot16BDailyData
from data_gateway_sdk.utils.buf_ref import BufRef


def test_smpm_ul_device_water_meter_nbiot_16b_daily() -> None:
    case_serialized = bytes.fromhex("850cc0ffff7fba90e4eab106e4aaaf6a")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=33554431.0), temperature=23, battery_volts=3.3, event_reset=False, event_low_battery_level=False, event_temperature_limits=True, direct_flow_volume=112323.3, reverse_flow_volume=2.788, event_magnet=False, rsrp=-15, rsrq=-9, snr=22, ecl=1) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=33554431.0), temperature=23, battery_volts=3.3, event_reset=False, event_low_battery_level=False, event_temperature_limits=True, direct_flow_volume=112323.3, reverse_flow_volume=112323.3, event_magnet=False, rsrp=-15, rsrq=-9, snr=22, ecl=1))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c000000007fe000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000000e000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c380000007fe000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000000e000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e0000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e0000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e0000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e00000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e0000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e0000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000001000ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000001000c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffffffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffffffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffff0fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ffff0fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1ff0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1ff0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1f00ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc070000007fe000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc0700000000e000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f0000007fe000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("85cc3f00000000e000000000ff1f00c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff0000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff0000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff0000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff0000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff0000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff0000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff0000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff0000000000f0ff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff0000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff0000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff0000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff0000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff0000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff0000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff0000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff0000000000f0ff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff0000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff0000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff0000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff0000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff0000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff0000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff0000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff0000000000f00f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff0000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff0000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff0000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff0000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff0000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff0000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff0000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff0000000000f00f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff000000000010f03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff000000000010f000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff000000000010003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff0000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff0000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff0000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff0000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff0000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff0000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff0000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff0000000000100000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff00000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff00000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff00000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff00000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff00000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff00000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff00000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff00000000ffffff3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff00000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff00000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff00000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff00000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff00000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff00000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff00000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff00000000ffffff00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff00000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff00000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff00000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff00000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff00000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff00000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff00000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff00000000ffff0f3f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff00000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff00000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff00000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff00000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff00000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff00000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff00000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff00000000ffff0f00")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff00000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff00000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff00000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff00000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff00000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff00000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff00000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff00000000ff1ff03f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff00000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff00000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff00000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff00000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff00000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff00000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff00000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff00000000ff1ff000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff00000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff00000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff00000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff00000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff00000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff00000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff00000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff00000000ff1f003f")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=43, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff00000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff00000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff00000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff00000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff00000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff00000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff00000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff00000000ff1f0000")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=4.095, event_magnet=True, rsrp=-140, rsrq=-19, snr=-20, ecl=0) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=8.191, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff0000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff0000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff0000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff0000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff0000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff0000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff0000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff0000000000f0ffff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff0000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff0000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff0000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff0000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff0000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff0000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff0000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff0000000000f0ffc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff0000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff0000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff0000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff0000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff0000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff0000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff0000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff0000000000f00fff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff0000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff0000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff0000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff0000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff0000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff0000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff0000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff0000000000f00fc0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-13, rsrq=-19, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff000000000010f0ff")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=43, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c00000000ffff000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c0000000080ff000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c38000000ffff000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("850c3800000080ff000000000010f0c0")  # overflow
    assert SmpmUlDeviceWaterMeterNbiot16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, reverse_flow_volume=0.0, event_magnet=True, rsrp=-140, rsrq=-4, snr=-20, ecl=3) == SmpmUlDeviceWaterMeterNbiot16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceWaterMeterNbiot16BDailyData.serialize(SmpmUlDeviceWaterMeterNbiot16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_low_battery_level=True, event_magnet=True, event_reset=True, event_temperature_limits=True, reverse_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
