from datetime import timedelta

from data_gateway_sdk.utils.buf_ref import BufRef

from data_gateway_sdk.protocols.smpm.smpm_ul_device_gaz_flow_32b_daily import SmpmUlDeviceGazFlow32BDailyData

def test_smpm_ul_device_gaz_flow_32b_daily() -> None:
    case_serialized = bytes.fromhex("840cc0ffff7fba90e4eab10623250a087c0db8010a000a00b61d862a0801e403")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=33554431.0), temperature=23, battery_volts=3.3, event_reset=False, event_low_battery_level=False, event_temperature_limits=True, direct_flow_volume=112323.3, direct_flow_volume_day_ago=3.5, reverse_flow_volume=10.98, event_battery_warn=True, event_system_error=False, event_flow_reverse=False, event_flow_speed_is_over_limit=False, event_sensor_error=False, event_sensor_error_temperature=False, event_case_was_opened=False, event_continuous_consumption=False, event_no_resource=True, earfcn=3452, pci=440, cell_id=10, tac=10, rsrp=42, rsrq=9, rssi=24, snr=22, band=8, ecl=1, tx_power=100, operation_mode=3) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=33554431.0), temperature=23, battery_volts=3.3, event_reset=False, event_low_battery_level=False, event_temperature_limits=True, direct_flow_volume=112323.3, direct_flow_volume_day_ago=3.5, reverse_flow_volume=112323.3, event_battery_warn=True, event_system_error=False, event_flow_reverse=False, event_flow_speed_is_over_limit=False, event_sensor_error=False, event_sensor_error_temperature=False, event_case_was_opened=False, event_continuous_consumption=False, event_no_resource=True, earfcn=3452, pci=440, cell_id=10, tac=10, rsrp=42, rsrq=9, rssi=24, snr=22, band=8, ecl=1, tx_power=100, operation_mode=3))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f00000000000000000000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f00000000000000000000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f00000000000000000000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f00000000000000000000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f00000000000000000000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f00000000000000000000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f00000000000000000000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f00000000000000000000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff0000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff0000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff0000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff0000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff0000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff0000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff0000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff0000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f00000000000000000000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f00000000000000000000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f00000000000000000000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f00000000000000000000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f00000000000000000000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f00000000000000000000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f00000000000000000000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f00000000000000000000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff0000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff0000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff0000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff0000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff0000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff0000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff0000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff0000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f00000000000000000000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f00000000000000000000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f00000000000000000000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f00000000000000000000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f00000000000000000000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f00000000000000000000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f00000000000000000000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f00000000000000000000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff0000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff0000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff0000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff0000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff0000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff0000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff0000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff0000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f00000000000000000000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f00000000000000000000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f00000000000000000000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f00000000000000000000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f00000000000000000000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f00000000000000000000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f00000000000000000000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f00000000000000000000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff0000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff0000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff0000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff0000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff0000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff0000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff0000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff0000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f00000000000000000000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f00000000000000000000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f00000000000000000000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f00000000000000000000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff0000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff0000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff0000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff0000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f00000000000000000000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f00000000000000000000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f00000000000000000000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f00000000000000000000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff0000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff0000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff0000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff0000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=0.0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff0000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff0000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000ff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000ff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffffff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffffff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000ff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000ff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffffff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff00000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffffff0000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=115, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000000000ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000000000ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff00ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff00ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000000000ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000000000ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff00ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff00ff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f00000000000000000000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f00000000000000000000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f00000000000000000000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f00000000000000000000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f00000000000000000000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f00000000000000000000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f00000000000000000000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f00000000000000000000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff0000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff0000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff0000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff0000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff0000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff0000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff0000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff0000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f00000000000000000000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f00000000000000000000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f00000000000000000000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f00000000000000000000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f00000000000000000000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f00000000000000000000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f00000000000000000000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f00000000000000000000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff0000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff0000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff0000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff0000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff0000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff0000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff0000ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff0000ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f00000000000000000000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f00000000000000000000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f00000000000000000000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f00000000000000000000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f00000000000000000000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f00000000000000000000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f00000000000000000000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f00000000000000000000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff0000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff0000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff0000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff0000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff0000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff0000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff0000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff0000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f00000000000000000000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f00000000000000000000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f00000000000000000000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f00000000000000000000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f00000000000000000000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f00000000000000000000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f00000000000000000000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f00000000000000000000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff0000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff0000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff0000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff0000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff0000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff0000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff0000ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff0000ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff000000ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff000000ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f00000000000000000000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f00000000000000000000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000000000000000000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f00000000000000000000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f00000000000000000000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000000000000000000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff0000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff0000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f000000000000ffff000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff0000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff0000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f000000000000ffff000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f00000000000000000000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f00000000000000000000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000000000000000000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f00000000000000000000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f00000000000000000000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000000000000000000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=0, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff0000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff0000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f000000000000ffff000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff0000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff0000000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f000000000000ffff000000000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=0, cell_id=0, tac=65535, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=0, reverse_flow_volume=81.92, rsrp=-140, rsrq=-20, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff00000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff00000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff00000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff00000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff00000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff00000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff00000000ffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff00000000ffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff0000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff0000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff0000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff0000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff0000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff0000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff0000ffffffffffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff0000ffffffffffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff00000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff00000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff00000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff00000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff00000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff00000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff00000000ffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff00000000ffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff0000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff0000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff0000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff0000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff0000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff0000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff0000ffffffffff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff0000ffffffffff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff00000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff00000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff00000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff00000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff00000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff00000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff00000000ffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff00000000ffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff0000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff0000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff0000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff0000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff0000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff0000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff0000ffffffff00ff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff0000ffffffff00ff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff00000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff00000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff00000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff00000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff00000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff00000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff00000000ffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff00000000ffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff0000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff0000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff0000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff0000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff0000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff0000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff0000ffffffff000000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff0000ffffffff00000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=235, rssi=-110, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=235, rssi=-110, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff00000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff00000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff00000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff00000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff00000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff00000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff00000000ff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff00000000ff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff0000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff0000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff0000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c380000007fe0000000000000f80f0000ffff0000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff0000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff0000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff0000ffffff00ffff00000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c3800000000e0000000000000f80f0000ffff0000ffffff00ffff0000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=235, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=235, sync_time_days_ago=timedelta(seconds=604800.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff00000000ff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff00000000ff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=0, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=0, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c000000007fe0000000000000f80f0000ffff0000ffffff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=92, timestamp_s=timedelta(seconds=134217728.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=-128))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffff00ff000000ff00")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=127, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=0.0), tx_power=127))  # noqa: E501
    case_serialized = bytes.fromhex("840c0000000000e0000000000000f80f0000ffff0000ffffff00ff0000000000")  # overflow
    assert SmpmUlDeviceGazFlow32BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, reverse_flow_volume=0.0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True, earfcn=0, pci=65535, cell_id=0, tac=65535, rsrp=115, rsrq=-20, rssi=145, snr=-20, band=0, ecl=0, tx_power=-128, operation_mode=0) == SmpmUlDeviceGazFlow32BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow32BDailyData.serialize(SmpmUlDeviceGazFlow32BDailyData(band=0, battery_volts=0.0, cell_id=0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, direct_flow_volume_day_ago=0.0, earfcn=0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, operation_mode=0, pci=65535, reverse_flow_volume=0.0, rsrp=115, rsrq=-20, rssi=145, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), tac=65535, temperature=-35, timestamp_s=timedelta(seconds=134217728.0), tx_power=-128))  # noqa: E501
