from datetime import timedelta

from data_gateway_sdk.utils.buf_ref import BufRef

from data_gateway_sdk.protocols.smpm.smpm_ul_device_gaz_flow_16b_daily import SmpmUlDeviceGazFlow16BDailyData


def test_smpm_ul_device_gaz_flow_16b_daily() -> None:
    case_serialized = bytes.fromhex("820cc0ffff7fba90e4eab1067d550b08")
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=33554431.0), temperature=23, battery_volts=3.3, event_reset=False, event_low_battery_level=False, event_temperature_limits=True, direct_flow_volume=112323.3, rsrp=-15, rsrq=-9, snr=22, ecl=1, event_battery_warn=True, event_system_error=False, event_flow_reverse=False, event_flow_speed_is_over_limit=False, event_sensor_error=False, event_sensor_error_temperature=False, event_case_was_opened=False, event_continuous_consumption=False, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=33554431.0), temperature=23, battery_volts=3.3, event_reset=False, event_low_battery_level=False, event_temperature_limits=True, direct_flow_volume=112323.3, rsrp=-15, rsrq=-9, snr=22, ecl=1, event_battery_warn=True, event_system_error=False, event_flow_reverse=False, event_flow_speed_is_over_limit=False, event_sensor_error=False, event_sensor_error_temperature=False, event_case_was_opened=False, event_continuous_consumption=False, event_no_resource=True))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c000000007fe0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000000e0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c380000007fe0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000000e0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e000000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e000000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e00000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e00000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e000000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e000000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e00000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e00000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc070000007fe0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000000e0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f0000007fe0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000000e0000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=0.0, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=0.0, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c00000000ffff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c0000000080ff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c38000000ffff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("820c3800000080ff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=0.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=0.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=0.0, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff00000000fffff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff00000000ff07f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000007ff8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000007f00f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff0000000080fff90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000008007f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff0000000000f8f90f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000000000f80f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=0, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=0, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff00000000ffffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff00000000ff07fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000007ff8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000007f00fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-13, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-13, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff0000000080ffff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000008007fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-4, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-4, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff0000000000f8ff0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=43, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=43, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc07000000ffff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc0700000080ff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=0.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=0.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f000000ffff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=92, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=92, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=0.0)))  # noqa: E501
    case_serialized = bytes.fromhex("82cc3f00000080ff000000000000fe0f")  # overflow
    assert SmpmUlDeviceGazFlow16BDailyData(days_ago=timedelta(seconds=2678400.0), sync_time_days_ago=timedelta(seconds=604800.0), timestamp_s=timedelta(seconds=0.0), temperature=-35, battery_volts=6.3, event_reset=True, event_low_battery_level=True, event_temperature_limits=True, direct_flow_volume=0.0, rsrp=-140, rsrq=-19, snr=-20, ecl=3, event_battery_warn=True, event_system_error=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_sensor_error=True, event_sensor_error_temperature=True, event_case_was_opened=True, event_continuous_consumption=True, event_no_resource=True) == SmpmUlDeviceGazFlow16BDailyData.parse(BufRef(case_serialized))  # noqa: E501
    assert case_serialized == SmpmUlDeviceGazFlow16BDailyData.serialize(SmpmUlDeviceGazFlow16BDailyData(battery_volts=6.3, days_ago=timedelta(seconds=2678400.0), direct_flow_volume=8589934.592, ecl=3, event_battery_warn=True, event_case_was_opened=True, event_continuous_consumption=True, event_flow_reverse=True, event_flow_speed_is_over_limit=True, event_low_battery_level=True, event_no_resource=True, event_reset=True, event_sensor_error=True, event_sensor_error_temperature=True, event_system_error=True, event_temperature_limits=True, rsrp=-140, rsrq=-19, snr=-20, sync_time_days_ago=timedelta(seconds=604800.0), temperature=-35, timestamp_s=timedelta(seconds=134217728.0)))  # noqa: E501
